/*
 *
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "caosdb/data_type.h"               // for DataType, AtomicDataType
#include "caosdb/entity/v1alpha1/main.pb.h" // for AtomicDataType, DataType
#include <gtest/gtest-message.h>            // for Message
#include <gtest/gtest-test-part.h>          // for TestPartResult, SuiteApi...
#include <gtest/gtest_pred_impl.h>          // for AssertionResult, Test
#include <memory>                           // for allocator

namespace caosdb::entity {
using ProtoEntity = caosdb::entity::v1alpha1::Entity;
using ProtoParent = caosdb::entity::v1alpha1::Parent;
using ProtoDataType = caosdb::entity::v1alpha1::DataType;
using ProtoAtomicDataType = caosdb::entity::v1alpha1::AtomicDataType;

TEST(test_data_type, test_atomic) {
  ProtoDataType proto_data_type;

  for (int i = 1; i < 6; i++) {
    proto_data_type.set_atomic_data_type(static_cast<ProtoAtomicDataType>(i));
    DataType data_type(&proto_data_type);

    EXPECT_FALSE(data_type.IsReference());
    EXPECT_FALSE(data_type.IsList());
    EXPECT_TRUE(data_type.IsAtomic());
    EXPECT_EQ(data_type.AsAtomic(), static_cast<AtomicDataType>(i));
  }
}

TEST(test_data_type, test_list_of_atomic) {
  ProtoDataType proto_data_type;
  auto *proto_list_data_type = proto_data_type.mutable_list_data_type();

  for (int i = 1; i < 6; i++) {
    proto_list_data_type->set_atomic_data_type(
      static_cast<ProtoAtomicDataType>(i));
    DataType data_type(&proto_data_type);

    EXPECT_FALSE(data_type.IsReference());
    EXPECT_FALSE(data_type.IsAtomic());
    EXPECT_TRUE(data_type.IsList());
    const auto &list_data_type = data_type.AsList();
    EXPECT_TRUE(list_data_type.IsListOfAtomic());
    EXPECT_EQ(list_data_type.GetAtomicDataType(),
              static_cast<AtomicDataType>(i));
  }
}

} // namespace caosdb::entity
